  #include "BuiltIns.h"

#include "BuiltIns.h"
#include "API.h"
#include "ifi_aliases.h"
//#include "user_defs.h"   
#include <p18cxxx.h>

// Define stepper motor constants here...
#define TRUE 1                                  // Boolean True value
#define FALSE 0                                 // Boolean false value
#define LOW  0                                  // Logic low
#define HIGH 1                                  // Logic high
#define AZIMUTH_STEPPER_MOTOR  0              	// Azimuth Stepper Motor ID
#define ELEVATION_STEPPER_MOTOR 1              	// Elevation Stepper Motor ID
#define CLOCKWISE           1                 	// Clockwise direction (CW) for stepper motor
#define COUNTER_CLOCKWISE    0                 	// Counterclockwise direction (CCW) for stepper motor

// Define Stepper Motor Control modes
#define HALF_STEP    0                         	// Half step control mode
#define FULL_STEP    1                         	// Full step control mode
#define ONE_PHASE    2                         	// One phase wave drive control mode
#define TWO_PHASE    3                         	// Two phase wave drive control mode

#define AzimuthEncoderInt        	1             	// Azimuth Encoder Interrupt
#define ElevationEncoderInt    		2               // Azimuth Encoder Interrupt

// Define Vex Controller Digital Output port assignments here...
#define AzimuthCCWLimit         	1               // Elevation CCW limit switch
//#define AzimuthCWLimit          	2               // Elevation CW limit switch
#define AzimuthCWLimit          	6               // Elevation CW limit switch
#define ElevationCCWLimit        	3               // Azimuth CCW limit switch
//#define ElevationCWLimit         	4               // Azimuth CW limit switch
#define AzimuthEncoderInput        	4               // Azimuth Encoder Input
#define ElevationEncoderInput    	5               // Azimuth Encoder Input

#define ElevationCWLimit         	7               // Azimuth CW limit switch

// Define Vex Controller Digital Output port assignments here...

#define ElevationDirection        	8      			// Direction for right stepper motor
#define ElevationStep             	9      			// Used to activate right stepper motor (pulse)
#define ElevationHalfStep        	10      		// Selects half step sequence (high)
#define ElevationOnePhase       	11      		// Selects one phase wave drive (high) or two phase (low)

#define AzimuthDirection        	12      		// Direction for left stepper motor
#define AzimuthStep             	13      		// Used to activate left stepper motor (pulse)
#define AzimuthHalfStep         	14      		// Selects half step sequence (high)
#define AzimuthOnePhase         	15      		// Selects one phase wave drive (high) or two phase (low)

// in the declaration section of your VEX PIC18 C application and re-compile the application...

#define Total_Commands   50 
int Azimuth_Stepper_Commands[Total_Commands] = {

  0, 37, 44, 50, 57, 63, 69, 74, 80, 85, 90, 95, 100, 104, 108, 112, 116, 119, 121, 124, 126, 128, 129, 130, 131, 131, 131, 130, 129, 128, 126, 124, 121, 119, 116, 112, 108, 104, 100, 95, 90, 85, 80, 74, 69, 62, 56, 50, 43, 37 };

int Elevation_Stepper_Commands[Total_Commands] = {

 0, 552, 560, 569, 577, 586, 595, 604, 614, 623, 633, 643, 653, 664, 675, 685, 696, 708, 719, 731, 743, 754, 766, 778, 791, 803, 815, 827, 839, 851, 863, 875, 886, 898, 909, 920, 931, 942, 952, 962, 973, 983, 992, 1002, 1011, 1019, 1028, 1037, 1046, 1054};


      
// Define Azimuth Stepper Controller variables here...
unsigned long int AzimuthStepperCount = 0;        	// Current of steps that the Azimuth Stepper motor has moved
int AzimuthDirectionValue = COUNTER_CLOCKWISE;      // Direction of Azimuth Stepper motor        
int AzimuthStepperMode = HALF_STEP;                // Control mode of Azimuth Stepper motor  
int AzimuthDuration = 10;                          // Delay between steps for Azimuth Stepper motor in milliseconds
int AzimuthCCWLimitValue = 0;                      // Elevation CCW Limit Switch value
int AzimuthCWLimitValue = 0;                       // Elevation CCW Limit Switch value
 
// Define Elevation Stepper Controller variables here...
unsigned long int ElevationStepperCount = 0;        // Current of steps that the Elevation Stepper motor has moved
int ElevationDirectionValue = COUNTER_CLOCKWISE;    // Direction of Elevation Stepper motor        
int ElevationStepperMode = HALF_STEP;               // Control mode of Elevation Stepper motor  
int ElevationDuration = 10;						    // Delay between steps for Elevation Stepper motor in milliseconds
int ElevationCCWLimitValue = 0;                     // Elevation CCW Limit Switch value
int ElevationCWLimitValue = 0;                      // Elevation CCW Limit Switch value

long AzimuthEncoderCounts = 0;						// Azimuth Encoder Counts 
long ElevationEncoderCounts = 0;						// Elevation Encoder Counts 

// Insert this code containing Azimuth and Elevation (Altitude) stepper commands


// Declare function prootypes here
void InitializeStepperMotors(void);
int CheckLimitSwitches(void);
int CalibrateAzimuthDrive (void);
int CalibrateElevationDrive (void);
void UpdateSteppers(void);
void MoveStepper(int StepperID, int Direction, int NumberSteps);
 
// declare global data here
char inchar;
long int Count = 0;

/*  
Vex Remote Control Experiment #

This application will run the Vex Remote Control Experiment by reading the joystick positions and switches
from the Vex RC Remote and sending them to the motors using the Operator Input (OI) functions from the WPILib.  
The motors are driven using Arcade2 style joystick controls.

Future upgrades include Motion Script Generation -  Record/Play capability by storing the joystick data to serial EEPROM or disk and reading it
back to the Vex Controller.

This function must be here for a competition project. It 
is automatically referenced by WPILib at startup and run.  
At that point the SetCompetitionMode function sets the  
competition mode. Basically, a mode of 0 is the default  
(without the IO_Initialization function) and runs a main  
function only.  SetCompetitionMode(1) runs a competition project as shown. 
#define PORT_1 1
#define PORT_2 2
#define CHANNEL_1 1
#define CHANNEL_2 2
#define CHANNEL_3 3
#define CHANNEL_4 4
#define CHANNEL_5 5
#define CHANNEL_6 6
#define CHANNEL_5_TOP 1
#define CHANNEL_5_BOTTOM 2
#define CHANNEL_6_TOP 3
#define CHANNEL_6_BOTTOM 4
*/ 

/***************************************************************************************
 * IO_Initialization -  This procedure initializes the Vex Controller and I/O Ports. 
 ***************************************************************************************/
void IO_Initialization(void)
{ 
	SetCompetitionMode(0,1); // VEX Runs main
  //SetCompetitionMode(1, 1); // VEX
 
   //Controller Port   #: 01,02,03,04,05,06,07,08,09,10,11,12,13,14,15,16 
   //DefineControllerIO(4, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0); 
   // The first parameter is the number of analog input ports.
   //                        01     02     03     04     05     06     06     07     08     09      10      11     13      14      15      16
   //DefineControllerIO ( 5, INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   //DefineControllerIO ( 2, INPUT ,INPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   DefineControllerIO ( 0, INPUT ,INPUT ,INPUT  ,INPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
 
 
} 
 

/***************************************************************************************
 *  Initialize - This function is run immediately when the robot is powered on 
 *  regardless of the field mode. 
 ***************************************************************************************/
void Initialize(void) 
{ 
#ifdef NEW_CODE
	int i; 
	for (i = 0; i < 10; i++) 
	{ 
		PrintToScreen("Initialize %d\r", i); 
		//printf("Initialize %d\r", i); 
		Wait(500); 
	}
#endif 
} 


/***************************************************************************************
 Autonomous - is run as soon as the field controls enable the 
 robot. At the end of the autonomous period, the Autonomous 
 function will end (note: even if it is in an infinite loop 
 as in the example, it will be stopped). 
 ***************************************************************************************/
void Autonomous(void) 
{
#ifdef NEW_CODE
 	while (1)
 	{ 
	PrintToScreen("In autonomous\r"); 
	//	printf("In autonomous\r"); 
		Wait(500); 
	} 
#endif
} 


/***************************************************************************************
 OperatorControl - The OperatorControl function will be called when the field 
 switches to operator mode. If the field ever switches back 
 to autonomous, then OperatorControl will automatically exit 
 and the program will transfer control to the Autonomous  function. 
 ***************************************************************************************/
 void OperatorControl(void) 
{ 
#ifdef NEW_CODE
	unsigned char port;
	unsigned char function;
 	unsigned char invert;
    unsigned char AzimuthPWM;
    unsigned char ElevationPWM;

	PrintToScreen("In OperatorControl...\r");
    //printf("In OperatorControl...\r"); 
	Wait(500); 
	TwoWheelDrive(1, 2); // define robot two wheels 

	while(1)
	{
		// Read Vex RC joystick positions 
 		OIToPWM(CHANNEL_1, 1,  AzimuthPWM , 0);
	    OIToPWM(CHANNEL_2, 2,  ElevationPWM, 0);
    	PrintToScreen("AzimuthPWM = %d, ElevationPWM = %d \r", AzimuthPWM, ElevationPWM); 

		// Move motors using updated joytick Arcade style positions 
		Arcade2(PORT_1,
                CHANNEL_1, CHANNEL_2,
                AzimuthPWM, ElevationPWM,
                0, 0);
	}
	while (1)
	{
		 PrintToScreen("In OperatorControl\r"); 
		 //printf("In OperatorControl\r"); 
		Wait(500); 
	} 
#endif

} 

/*  the main program is not used, but needs to be here to
  statisfy a reference to it. This requirement will probably 
go away in the next version of WPILib. 
Vex receiver to controller port numbers #define PORT_1 1 #define PORT_2 2
 // Vex transmitter channel numbers
 #define CHANNEL_1 1 
#define CHANNEL_2 2 
#define CHANNEL_3 3 
#define CHANNEL_4 4 
#define CHANNEL_5 5 
#define CHANNEL_6 6 
#define CHANNEL_5_TOP 1 
#define CHANNEL_5_BOTTOM 2 
#define CHANNEL_6_TOP 3 
#define CHANNEL_6_BOTTOM 4

// OI values that are send directly to motors or digital outputs 

void OIToDOutput(unsigned char port, unsigned char function, unsigned char dport); 
void OIToPWM(unsigned char port, unsigned char function, unsigned char pwm, unsigned char invert);

void Arcade2(unsigned char port,
                unsigned char moveChannel, unsigned char rotateChannel,
                unsigned char leftPWM, unsigned char rightPWM,
                unsigned char leftInvert, unsigned char rightInvert);
*/ 

/***************************************************************************************
 * InitializeStepperMotors -  Initialize Azimuth and Elevation Stepper Motor Controllers.
 ***************************************************************************************/
void InitializeStepperMotors(void)
{
    // Initialize Azimuth Stepper Controller variables here...
    AzimuthStepperCount = 1000;        		   // Number of steps to move the Azimuth Stepper motor
    AzimuthDirectionValue = COUNTER_CLOCKWISE;            // Direction of Azimuth Stepper motor        
    AzimuthStepperMode = FULL_STEP;               // Control mode of Azimuth Stepper motor  

    // Initialize Elevation Stepper Controller variables here...
    ElevationStepperCount = 1000;                     // Number of steps to move the Elevation Stepper motor
    ElevationDirectionValue = COUNTER_CLOCKWISE;           // Direction of Elevation Stepper motor        
    ElevationStepperMode = FULL_STEP;              // Control mode of Elevation Stepper motor  


    // Perform a simple Stepper Motor test on both the left and right stepper motors by having them move 360 steps in a clockwise direction and
    // then reverse direction 

    // Stepper motor test
    //void SetDigitalOutput(unsigned char port, unsigned char value);
    //unsigned char GetDigitalInput(unsigned char port);
    //void SetDirection(unsigned char port, unsigned char direction);

 	SetDirection(ElevationCCWLimit , INPUT);             			// Direction for Elevation CCW limit switch
 	SetDirection(ElevationCWLimit , INPUT);             			// Direction for Elevation CW limit switch
 	SetDirection(AzimuthCCWLimit , INPUT);             			// Direction for Azimuth CCW limit switch
 	SetDirection(AzimuthCWLimit , INPUT);             		     	// Direction for Azimuth CW limit switch
         
	// Set the Vex Controller Digital I/O to Outputs assignments here...
	SetDirection(AzimuthDirection, OUTPUT);             			// Direction for left stepper motor
	SetDirection(AzimuthStep, OUTPUT);     						// Used to activate left stepper motor (pulse)
	SetDirection(AzimuthHalfStep, OUTPUT);       					// Selects half step sequence (high)
	SetDirection(AzimuthOnePhase, OUTPUT);     					// Selects one phase wave drive (high) or two phase (low)

	SetDirection(ElevationDirection, OUTPUT);      					// Direction for right stepper motor
	SetDirection(ElevationStep, OUTPUT);     						// Used to activate right stepper motor (pulse)
	SetDirection(ElevationHalfStep, OUTPUT);      					// Selects half step sequence (high)
	SetDirection(ElevationOnePhase, OUTPUT);      					// Selects one phase wave drive (high) or two phase (low)


#ifdef TEST


// Test the digital outputs to make sure that they are working by generating square waves on all digital output 
// pins used to control the two stepper motors
while (1)
{

 // Test  left direction motor by toggling the direction bit IO03..IO08
    SetDigitalOutput(AzimuthDirection, HIGH);
    SetDigitalOutput(AzimuthStep, HIGH);
    SetDigitalOutput(AzimuthHalfStep, HIGH);
    SetDigitalOutput(AzimuthOnePhase, HIGH);
    Wait(AzimuthDuration);

    SetDigitalOutput(AzimuthDirection, LOW);
    SetDigitalOutput(AzimuthStep, LOW);
    SetDigitalOutput(AzimuthHalfStep, LOW);
    SetDigitalOutput(AzimuthOnePhase, LOW);
    Wait(AzimuthDuration);

    // Test right control bits stepper motor by toggling the right direction bit IO08..IO11
    SetDigitalOutput(ElevationDirection, HIGH);
    SetDigitalOutput(ElevationStep, HIGH);
    SetDigitalOutput(ElevationHalfStep, HIGH);
    SetDigitalOutput(ElevationOnePhase, HIGH);
    Wait(ElevationDuration);

    SetDigitalOutput(ElevationDirection, LOW);
    SetDigitalOutput(ElevationStep, LOW);
   	SetDigitalOutput(ElevationHalfStep, LOW);
    SetDigitalOutput(ElevationOnePhase, LOW);
    Wait(ElevationDuration);

}
#endif

	// Start with left and right step set to low
	SetDigitalOutput(AzimuthStep, LOW);
	SetDigitalOutput(ElevationStep, LOW);
 
	// Set the Azimuth Stepper Control Mode here
	switch(AzimuthStepperMode)
	{
		case  ONE_PHASE:
		{
			// One phase mode
			SetDigitalOutput(AzimuthHalfStep, LOW);
			SetDigitalOutput(AzimuthOnePhase, HIGH);

			SetDigitalOutput(ElevationHalfStep, LOW);
			SetDigitalOutput(ElevationOnePhase, HIGH);
			break;
		}

		case  TWO_PHASE:
		{
			// Two phase mode
			SetDigitalOutput(AzimuthHalfStep, LOW);
			SetDigitalOutput(AzimuthOnePhase, LOW);

			SetDigitalOutput(ElevationHalfStep, LOW);
			SetDigitalOutput(ElevationOnePhase, LOW);
			break;
		}

		case  HALF_STEP:
		{
			// Half step mode
			SetDigitalOutput(AzimuthHalfStep, HIGH);
			SetDigitalOutput(AzimuthOnePhase, LOW);

			SetDigitalOutput(ElevationHalfStep, HIGH);
			SetDigitalOutput(ElevationOnePhase, LOW);
			break;
		}

		case  FULL_STEP:
		{
			// Full step mode
            break;
        }

        default:
        {
            break;
        }
	}


	// Set the Elevation Stepper Control Mode here
	switch(ElevationStepperMode)
	{
		case  ONE_PHASE:
		{
			// One phase mode
 			SetDigitalOutput(ElevationHalfStep, LOW);
			SetDigitalOutput(ElevationOnePhase, HIGH);
			break;
		}

		case  TWO_PHASE:
		{
			// Two phase mode
 			SetDigitalOutput(ElevationHalfStep, LOW);
			SetDigitalOutput(ElevationOnePhase, LOW);
			break;
		}

		case  HALF_STEP:
		{
			// Half step mode
 			SetDigitalOutput(ElevationHalfStep, HIGH);
			SetDigitalOutput(ElevationOnePhase, LOW);
			break;
		}

		case  FULL_STEP:
		{
			// Full step mode
            break;
        }

        default:
        {
            break;
        }
	}

	// Set initial direction for left and right stepper motors here
	SetDigitalOutput(AzimuthDirection,AzimuthDirectionValue);
	SetDigitalOutput(ElevationDirection, ElevationDirectionValue);


	// Initialize left stepper motor parameters
	AzimuthStepperCount = 0;
	AzimuthDuration = 5;

	// Initialize right stepper motor parameters
	ElevationStepperCount = 0;
	ElevationDuration = 5;
}

/***************************************************************************************
 * CheckLimitSwitches -  This procedure checks the states of the four limit switches and 
 * returns TRUE if all limit switches are open and returns FALSE if any one of the four 
 * switches is closed. 
 ***************************************************************************************/
int CheckLimitSwitches(void)
{
    int LocalSwitchValue = 0;           // Local switch value is the boolean AND of all four switch results

    ElevationCCWLimitValue = GetDigitalInput(ElevationCCWLimit);                    // Elevation CCW Limit Switch value
    ElevationCWLimitValue =  GetDigitalInput(ElevationCWLimit);                     // Elevation CW Limit Switch value
    AzimuthCCWLimitValue =  GetDigitalInput(AzimuthCCWLimit);                     // Azimuth CCW Limit Switch value
    AzimuthCWLimitValue =   GetDigitalInput(AzimuthCWLimit);                      // Azimuth CW Limit Switch value
    LocalSwitchValue = ElevationCCWLimitValue &  ElevationCWLimitValue & AzimuthCCWLimitValue & AzimuthCWLimitValue;           // Local switch value is the boolean OR of all four switch results

    return LocalSwitchValue;
}

 
/***************************************************************************************
 * CalibrateAzimuthDrive -  This procedure calibrates and aligns the Azimuth Stepper Motor 
 * drive. It does this by finding 0 degrees Azimuth  (backside of  Azimuth limit switch) and counts the 
 * number of steps counterclockwise until the Azimuth Limit Switch is triggered.
 * It returns a TRUE if successful. 
 ***************************************************************************************/
int CalibrateAzimuthDrive (void)
{ 

int TotalNumberAzimuthSteps = 0;   // Total number of steps corresponding to 360 degrees 

    PrintToScreen("trace 1: Start of Azimuth Drive Calibration \r");
    //PrintToScreen("1. Position the Sunbot platform clockwise to 0 degrees Azimuth (backside of  Azimuth limit switch).  \r");

    // Move The Azimuth Stepper Motors CounterClockwise 1 step at a time while checking the CCW Limit Switch state.    
    //while(GetDigitalInput(AzimuthCWLimit) == 1)
    while(GetDigitalInput(AzimuthCCWLimit) == 1)
    {
       	TotalNumberAzimuthSteps++;
       	//PrintToScreen("2. TotalNumberAzimuthSteps =  %d \r", TotalNumberAzimuthSteps);
 	   	//MoveStepper(AZIMUTH_STEPPER_MOTOR, CLOCKWISE, 1);
 	   	MoveStepper(AZIMUTH_STEPPER_MOTOR, COUNTER_CLOCKWISE, 1);
       	//Wait(100);

    	// Get the latest encoder reading from the Azimutn Drive
       	AzimuthEncoderCounts = - GetQuadEncoder(AzimuthEncoderInt,  AzimuthEncoderInput);
    	PrintToScreen("TotalNumberAzimuthSteps =  %d \r", TotalNumberAzimuthSteps);
    	PrintToScreen("AZ Encoder Counts       =  %ld\r", AzimuthEncoderCounts);
     }

    PrintToScreen("2. TotalNumberAzimuthSteps =  %d \r", TotalNumberAzimuthSteps);
    PrintToScreen("   AzimuthEncoderCounts    =  %ld \r", AzimuthEncoderCounts);

    PrintToScreen("trace 3: End of Azimuth Drive Calibration \r");
    return TRUE;
}

/***************************************************************************************
 * CalibrateElevaionDrive -  This procedure calibrates and aligns the Elevaion Stepper Motor 
 * drive. It does this by finding 0 degrees Elevation (backside of Elevation limit switch) and counts the 
 * number of steps counterclockwise until the Elevation Limit Switch is triggered.
 ***************************************************************************************/
int CalibrateElevationDrive (void)
{
 int TotalNumberElevationSteps = 0;   // Total number of steps corresponding to 360 degrees 

   PrintToScreen("trace 1: Start of Elevation Drive Calibration \r");
    //PrintToScreen("1. Position the solar panel platform clockwise to 0 degrees  Elevation (backside of Elevation limit switch).  \r");


    // Move The Elevation Stepper Motors Clockwise 1 step at a time while checking the CW Limit Switch state.    
    //while(GetDigitalInput(ElevationCWLimit) == 1)
    while(GetDigitalInput(ElevationCCWLimit) == 1)
    {
        TotalNumberElevationSteps++;    // Total number of steps corresponding to +/-90 degrees Elevation
        //PrintToScreen("2. TotalNumberElevationSteps =  %d \r", TotalNumberElevationSteps);

  	   	//MoveStepper(ELEVATION_STEPPER_MOTOR, CLOCKWISE, 1);
  	   	MoveStepper(ELEVATION_STEPPER_MOTOR, COUNTER_CLOCKWISE, 1);
       	//Wait(100);

     	// Get the latest encoder reading from the Elevation Drive
       	ElevationEncoderCounts = GetQuadEncoder(ElevationEncoderInt, ElevationEncoderInput);
    	PrintToScreen("TotalNumberElevationSteps =  %d \r", TotalNumberElevationSteps);
    	PrintToScreen("El Encoder Counts         =  %ld\r", ElevationEncoderCounts);
        //Wait(100);
    }
    PrintToScreen("2. TotalNumberElevationSteps =  %d \r", TotalNumberElevationSteps);
    PrintToScreen("   ElevationEncoderCounts    =  %ld \r", AzimuthEncoderCounts);

    PrintToScreen("trace 3: End of Elevation Drive Calibration \r");

    return TRUE;
}
 

/***************************************************************************************
 * MoveStepper -  This procedure moves the selected stepper motor in the direction specified (CW, CCW). 
 ***************************************************************************************/
void MoveStepper(int StepperID, int Direction, int NumberSteps)
{
   int i; // Loop index

   if (Direction == CLOCKWISE)
   {
      if (StepperID == AZIMUTH_STEPPER_MOTOR)
      {
  		// Set Azimuth Stepper direction Clockwise (CW)
  		AzimuthDirectionValue = CLOCKWISE;
  		SetDigitalOutput(AzimuthDirection, AzimuthDirectionValue);

        for (i=0; i<NumberSteps; i++)
        {
            // Check limit switches to see if any of them have been triggered.
            //if (GetDigitalInput(AzimuthCWLimit))
            if (GetDigitalInput(AzimuthCCWLimit))
            //if (CheckLimitSwitches())
            {
            	// Move Azimuth Stepper motor by toggling the step bit
       	        SetDigitalOutput(AzimuthStep, HIGH);
       	        Wait(AzimuthDuration);
    	        SetDigitalOutput(AzimuthStep, LOW);
    	        Wait(AzimuthDuration);
            }
            else
            {
                PrintToScreen("Warning - Reached Azimuth Drive CW Limit!!!\r");
                break;
            }
        }
      }
      else
      {
  		//Set Elevation Stepper direction Clockwise (CW)
  		ElevationDirectionValue = CLOCKWISE;
  		SetDigitalOutput(ElevationDirection, ElevationDirectionValue);

        for (i=0; i<NumberSteps; i++)
        {
            // Check limit switches to see if any of them have been triggered.
            //if (GetDigitalInput(ElevationCWLimit))
            if (GetDigitalInput(ElevationCCWLimit))
            //if (CheckLimitSwitches())
            {
       	       // Move Elevation Stepper motor by toggling the step bit
    	       SetDigitalOutput(ElevationStep, HIGH);
    	       Wait(ElevationDuration);
       	       SetDigitalOutput(ElevationStep, LOW);
    	       Wait(ElevationDuration);
            }
            else
            {
                PrintToScreen("Warning - Reached Elevation Drive CW Limit!!!\r");
                break;
            }
        }
      }
   }
   else
   {
      if (StepperID == AZIMUTH_STEPPER_MOTOR)
      {
  		//Set Azimuth Stepper direction Clockwise (CCW)
  		AzimuthDirectionValue = COUNTER_CLOCKWISE;
  		SetDigitalOutput(AzimuthDirection, AzimuthDirectionValue);

        for (i=0; i<NumberSteps; i++)
        {
            // Check limit switches to see if any of them have been triggered.
            if (GetDigitalInput(AzimuthCCWLimit))
            //if (CheckLimitSwitches())
            {
             	// Move Azimuth Stepper motor by toggling the step bit
       	        SetDigitalOutput(AzimuthStep, HIGH);
       	        Wait(AzimuthDuration);
    	        SetDigitalOutput(AzimuthStep, LOW);
    	        Wait(AzimuthDuration);
            }
            else
            {
                PrintToScreen("Warning - Reached Azimuth Drive CCW Limit!!!\r");
                break;
            }
        }
      }
      else
      {
  	  	 //Set Elevation Stepper direction Clockwise (CCW)
  		 ElevationDirectionValue = COUNTER_CLOCKWISE;
 		 SetDigitalOutput(ElevationDirection, ElevationDirectionValue);
 
         for (i=0; i<NumberSteps; i++)
         {
             // Check limit switches to see if any of them have been triggered.
            if (GetDigitalInput(ElevationCCWLimit))
            //if (CheckLimitSwitches())
            {
      	       // Move Elevation Stepper motor by toggling the step bit
    	       SetDigitalOutput(ElevationStep, HIGH);
    	       Wait(ElevationDuration);
       	       SetDigitalOutput(ElevationStep, LOW);
    	       Wait(ElevationDuration);
            }
            else
            {
                PrintToScreen("Warning - Reached Elevation Drive CCW Limit!!!\r");
                break;
            }
        }
      }   
   }

 
}
 
/***************************************************************************************
 * UpdateSteppers -  This procedure updates the stepper motors. 
 ***************************************************************************************/
void UpdateSteppers(void)
{
    PrintToScreen("trace 1: AzimuthStepperCount = %ld,  ElevationStepperCount = %ld\r", AzimuthStepperCount, ElevationStepperCount);

   // Check the steper motor counts to make sure they are not zero
    if (AzimuthStepperCount > 0)
    {
    	// Move Azimuth Stepper motor by toggling the step bit
    	SetDigitalOutput(AzimuthStep, HIGH);
    	Wait(AzimuthDuration);
    	SetDigitalOutput(AzimuthStep, LOW);
    	Wait(AzimuthDuration);
    	AzimuthStepperCount--;   
    }

    if (ElevationStepperCount > 0)
    {
    	// Move Elevation Stepper motor by toggling the step bit
    	SetDigitalOutput(ElevationStep, HIGH);
    	Wait(ElevationDuration);
    	SetDigitalOutput(ElevationStep, LOW);
    	Wait(ElevationDuration);
   		ElevationStepperCount--;  
    }
}

/***************************************************************************************
 * main -  This is the main procedure which initializes and  updates the stepper motors. 
 ***************************************************************************************/
void main(void)
{ 	
	unsigned char port;
	unsigned char function;
	unsigned char pwm;
	unsigned char invert;
    unsigned char AzimuthPWM;
    unsigned char ElevationPWM;
    unsigned int Joy_X_1; 
    unsigned int Joy_Y_1;
    unsigned int Joy_X_2;
    unsigned int Joy_Y_2;
    int i;

	// Open serial ports here
	//OpenSerialPortOne(BAUD_9600);
	//OpenSerialPortTwo(BAUD_9600);
	OpenSerialPortOne(BAUD_115200);
	OpenSerialPortTwo(BAUD_115200);

    // Set digital I/O port directions and initial values
    IO_Initialization();

    i=0;
 
    // Step 1 -Test the Azimuth Drive Vex Quadrature Optical Encoder.  Works!!!
    StartQuadEncoder(AzimuthEncoderInt,  AzimuthEncoderInput, 0);

    // Test the Elevation Drive Vex Quadrature Optical Encoder.  Works!!!
    StartQuadEncoder(ElevationEncoderInt, ElevationEncoderInput, 0);

    // Prest the Quadrature Encoders with zero
    PresetQuadEncoder(AzimuthEncoderInt,  AzimuthEncoderInput, 0);
    PresetQuadEncoder(ElevationEncoderInt, ElevationEncoderInput, 0);

#ifdef NEW_CODE
    while(1)
    {
      	// Get the latest encoder reading from the Elevation Drive
       	AzimuthEncoderCounts = GetQuadEncoder(AzimuthEncoderInt,  AzimuthEncoderInput);
       	ElevationEncoderCounts = GetQuadEncoder(ElevationEncoderInt, ElevationEncoderInput);
    	PrintToScreen("AZ Encoder Counts  =  %ld, EL Encoder Counts  =  %ld\r", AzimuthEncoderCounts, ElevationEncoderCounts);
        Wait(500);
    }

    // Stop reading the Azimuth and Elevation Drive Quadrature Encoders 
    StopQuadEncoder(AzimuthEncoderInt,  AzimuthEncoderInput);
    StopQuadEncoder(ElevationEncoderInt, ElevationEncoderInput);

#endif

     // Initialize Azimuth and Elevation Stepper Motor Controllers.
    InitializeStepperMotors();

#ifndef STEPPER_TEST
     PrintToScreen("Moving Azimuth stepper motor...\r");

     // Move The Azimuth Stepper Motors.  Works!!!
     while (1)
     {
        // Move 90 degrees Azimuth 
     	MoveStepper(AZIMUTH_STEPPER_MOTOR, CLOCKWISE, 400);
        Wait(5000);
     	MoveStepper(AZIMUTH_STEPPER_MOTOR, COUNTER_CLOCKWISE, 400);
        Wait(5000);

        //  Move 180 degrees Azimuth 
      	MoveStepper(AZIMUTH_STEPPER_MOTOR, CLOCKWISE, 800);
        Wait(5000);
      	MoveStepper(AZIMUTH_STEPPER_MOTOR, COUNTER_CLOCKWISE, 800);
        Wait(5000);

        //  Move 270 degrees Azimuth 
     	MoveStepper(AZIMUTH_STEPPER_MOTOR, CLOCKWISE, 1200);
        Wait(5000);
     	MoveStepper(AZIMUTH_STEPPER_MOTOR, COUNTER_CLOCKWISE, 1200);
        Wait(5000);

     }

     PrintToScreen("Moving Elevation stepper motor...\r");

     // Move The Elevation Stepper Motors.   Works!!! 
     while (1)
    {
     	MoveStepper(ELEVATION_STEPPER_MOTOR, COUNTER_CLOCKWISE, 800);
     	MoveStepper(ELEVATION_STEPPER_MOTOR, CLOCKWISE, 800);
    }

#endif

#ifdef TRACK_SUN

    PrintToScreen("Tracking the Sun...\r");

    // Track the Sun using the Azimuth and Elevation data from the astronmical navigation tables 
	while(1)
	{
		for (i=1; i<Total_Commands; i++)
		{
            PrintToScreen("Time interval: %d\r", i);
      
        	// Move the delta number of degrees Azimuth required for current udate time interval clokwise
    		MoveStepper(AZIMUTH_STEPPER_MOTOR, CLOCKWISE, (Azimuth_Stepper_Commands[i] - Azimuth_Stepper_Commands[i-1]));
     
        	// Move the delta number of degrees Elevation required for current update time interval counter-clockwise 
			MoveStepper(ELEVATION_STEPPER_MOTOR, COUNTER_CLOCKWISE, (Elevation_Stepper_Commands[i] - Elevation_Stepper_Commands[i-1]));

			// Wait for the next 6 second update interval for testing
            Wait(6000);

			// Wait for the next 10 minute update interval for normal operation
            //Wait(600000);
		}
	}

#endif

#ifdef MOVE_AZIMUTH
     PrintToScreen("Moving Azimuth stepper motor...\r");

     // Move The Azimuth Stepper Motors.  Works!!!
     while (1)
     {
        // Move 90 degrees Azimuth 
    	MoveStepper(AZIMUTH_STEPPER_MOTOR, CLOCKWISE, 921);
     	MoveStepper(AZIMUTH_STEPPER_MOTOR, COUNTER_CLOCKWISE, 921);
     }

#endif


#ifdef MOVE_ELEVATION
    PrintToScreen("Moving Elevation stepper motor...\r");

     // Move The Elevation Stepper Motors.   Works!!! 90 degrees
     while (1)
    {
     	MoveStepper(ELEVATION_STEPPER_MOTOR, CLOCKWISE, 766);
     	MoveStepper(ELEVATION_STEPPER_MOTOR, COUNTER_CLOCKWISE, 766);
    }
#endif
  
 

#ifdef CALIBRATE_ELEVATION
    // Calibrate and align the Elevation Stepper Motor drive
    CalibrateElevationDrive();

    // Wait indefinitely
    while(1);

 	// Wait a bit to take down Elevation  calibration data results
    Wait(9000);
#endif

#ifdef CALIBRATE_AZIMUTH
     // Calibrate and align the Azimuth Stepper Motor drive
    CalibrateAzimuthDrive();

	// Wait a bit to take down Azimuth calibration data results
    //Wait(9000);
  
    // Wait indefinitely
    while(1);

#endif



#ifdef REGISTER_REPEATING_TIMER_TEST

while(1)
{

   PrintToScreen("AzimuthStepperCount = %ld  ElevationStepperCount = %ld \r",AzimuthStepperCount, ElevationStepperCount);

   if (AzimuthStepperCount == 0)  
   {

   		// Reverse direction
  		AzimuthDirectionValue = ~AzimuthDirectionValue;
  		ElevationDirectionValue = ~ElevationDirectionValue;
  		SetDigitalOutput(AzimuthDirection, AzimuthDirectionValue);
  		SetDigitalOutput(ElevationDirection, ElevationDirectionValue);
   		AzimuthStepperCount = 360;
   		ElevationStepperCount = 360;
    }

    // Move left stepper motor by toggling the step bit
    UpdateSteppers();
 
  }


    // Initialize repeating timers used to update stepper motors every n milliseconds using a repeating timer
    RegisterRepeatingTimer(1000, UpdateSteppers);
    while(1)
    {
      //PrintToScreen("TestRun\r\r");
    }

#endif
 

#ifdef TEST

// Check the limit switches and print their state.  Works!!!
while(1)
{
   // Check the limit switches and print their state
   CheckLimitSwitches(); 
   PrintToScreen("ElevationCWLimitValue = %d  AzimuthCWLimitValue = %d \r",ElevationCWLimitValue, AzimuthCWLimitValue);

}

stepperLoop:

   PrintToScreen("AzimuthStepperCount = %ld  ElevationStepperCount = %ld \r",AzimuthStepperCount, ElevationStepperCount);

   if (AzimuthStepperCount > 360)  
   {

   		// Reverse direction
  		AzimuthDirectionValue = ~AzimuthDirectionValue;
  		ElevationDirectionValue = ~ElevationDirectionValue;
  		SetDigitalOutput(AzimuthDirection, AzimuthDirectionValue);
  		SetDigitalOutput(ElevationDirection, ElevationDirectionValue);
   		AzimuthStepperCount = 0;
   		ElevationStepperCount = 0;
    }

    // Move left stepper motor by toggling the step bit
    SetDigitalOutput(AzimuthStep, HIGH);
    Wait(AzimuthDuration);
    SetDigitalOutput(AzimuthStep, LOW);
    Wait(AzimuthDuration);

    // Move right stepper motor by toggling the step bit
    SetDigitalOutput(ElevationStep, HIGH);
    Wait(ElevationDuration);
    SetDigitalOutput(ElevationStep, LOW);
    Wait(ElevationDuration);

    AzimuthStepperCount++;   
    ElevationStepperCount++;  

    goto StepperLoop;


	// Serial port test.  Works using WPILib!!!
	while(1)
	{
		PrintToScreen("This is a test %d times! \r", i++);    // Works using WPILib!!!
		//WriteSerialPortOne(0xAA);
		//WriteSerialPortTwo(0xAA);
        Wait(10);
	}

	while(1)
	{
		 
		WriteSerialPortTwo("B");
	}

	while(1)
	{
		WriteSerialPortOne("A");
		WriteSerialPortTwo("B");
		for (i=0; i<1000; i++)
		{
			printf("This is a test %d times UART1 \n", i);
		}

	}

	while (1)
	{
		// Read data from the serial port
		inchar = ReadSerialPortOne();
		//inchar = ReadSerialPortTwo();

		if (inchar != 0)
		{
			// valid character is available so echo it back
			PrintToScreen("inchar = !%c! \r", inchar);
		}
	}
#endif

#ifdef NEW_CODE

	// This code works!!!
  	TwoWheelDrive(1, 2); // define robot two wheels 

	while (1)
	{
		// Get data from first joystick (right)  Works!!!
  		Joy_X_1 = GetOIAInput(PORT_1, CHANNEL_1);
		Joy_Y_1 = GetOIAInput(PORT_1, CHANNEL_2);

		// Get data from second joystick (left)  Works!!!
   		Joy_X_2 = GetOIAInput(PORT_1, CHANNEL_3);
		Joy_Y_2 = GetOIAInput(PORT_1, CHANNEL_4);

    	PrintToScreen("Joy_X_1 = %d, Joy_Y_1 = %d, Joy_X_2 = %d, Joy_Y_2 = %d \r", Joy_X_1, Joy_Y_1, Joy_X_2, Joy_Y_2 ); 
		//Wait(500);


		// Move the selected motors.  Works!!!
		SetPWM (1,Joy_X_1);
		SetPWM (2,Joy_Y_1);		
        SetPWM (3,Joy_X_2);
		SetPWM (4,Joy_Y_2);

 /*
		// Move motors using updated joytick Arcade style positions 
		Arcade2(PORT_1,
                CHANNEL_1, CHANNEL_2,
                AzimuthPWM, ElevationPWM,
                0, 0);
*/

	}
/*
An earlier search didn't turn up anything on SetMotor() or SetServo() but a second search turned up some examples using SetPWM(). 
I believe my problem has to do with the size of the variable I was using as an argument. Things seem to be working if I use:

unsigned char armASpeed = 0;
while(1) {
armASpeed = GetRxInput(1,3);
PrintToScreen("armASpeed: %d\n",(int)armASpeed);
SetPWM(8, armASpeed);
}

*/
	while(1)
	{
		// Read Vex RC joystick positions 
 		OIToPWM(PORT_1, 1,  AzimuthPWM , 0);
	    OIToPWM(PORT_1, 2,  ElevationPWM, 0);
    	PrintToScreen("AzimuthPWM = %d, ElevationPWM = %d \r", AzimuthPWM, ElevationPWM); 

		// Move motors using updated joytick Arcade style positions 
		Arcade2(PORT_1,
                CHANNEL_1, CHANNEL_2,
                AzimuthPWM, ElevationPWM,
                0, 0);
	}


	TwoWheelDrive(1, 2); // define robot two wheels 
	StartUltrasonic(1, 7); // start ultrasonic rangefinder 
	while (GetUltrasonic(1, 7) > 24) // loop until about 2 feet away 
	{ 
		Drive(100, 0); // drive forward } 
		StopUltrasonic(1, 7); // turn off rangefinder 
		Drive(0, 0); // stop driving
	 }
#endif

}
